#pragma once

#include "ActorInterface_Private.h"

#if ! defined(WIN32)
#include <libkern/OSAtomic.h>
#endif

/////////////////////////////////////////////
#include "incconferenceintf.h"
#include "INpMsgrP2P.h"
/////////////////////////////////////////////


class CProfileImageActor;

class CProfileImage : public IGTProfileImage
#ifdef __USE_PROFILE_IMG_BY_P2P__
	, public _INpStreamSessionEvents
#endif
	, public IGTWASActorEventPrivate
{
public:
	CProfileImage(CProfileImageActor *pActor);
	virtual ~CProfileImage(void);

	// For GTObject
protected:	
	virtual void OnReleaseObject(void) { delete this; }
	
public:		
	bool CanDelete();

	inline bool IsReady(void)
	{
		return m_bReady;
	}

	virtual LPCWSTR GetUserID() const
	{
		return m_sData.strUserID.c_str();
	}

	virtual LPCWSTR GetFileName() const
	{
		return m_sData.strFileName.c_str();
	}
	
	virtual DWORD GetCRC() const
	{
		return m_sData.dwCRC;
	}

	virtual void SetFileName(LPCWSTR strFileName)
	{
		m_sData.strFileName = strFileName;
	}

	void SetData(const P_GT_PROFILE_IMAGE_DATA &sData);
	void SetData(const GT_PROFILE_IMAGE_DATA &sData);
	const P_GT_PROFILE_IMAGE_DATA & GetData();

	bool RequestGetImage(void);
	bool CheckImage(void);

private:
	bool RequestGetImageFromWAS(void);
#ifdef __USE_PROFILE_IMG_BY_P2P__	
	bool CopyFromTempFile(void);
#endif	

protected:
#ifdef __USE_PROFILE_IMG_BY_P2P__	
	// _INpStreamSessionEvents 
	virtual void CALLBACK NpStreamSession_OnConnect(INpStreamSession *pSession, int nResultCode, int nErrorCode);
	virtual void CALLBACK NpStreamSession_OnClose(INpStreamSession *pSession, int nSocketError, bool bRecvInvalidPacket);
	virtual void CALLBACK NpStreamSession_OnRecvPacket(INpStreamSession *pSession, INpPacket * pPacket);
	virtual void CALLBACK NpStreamSession_OnSendReady(INpStreamSession * /*pSession*/);
#endif
	// IGTWASActorEvent 
	virtual bool OnGTWASActor_ResponseProfileImage(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, GTProfileImgError eProfileError, bool bUploadFile, const GT_PROFILE_IMAGE_DATA& sData);

private:
	bool m_bWorking;
	bool m_bReady;

	trid_t m_tridDownloadFromWAS;
	std::wstring m_strTempFile;
	HANDLE m_hTempFile;

	P_GT_PROFILE_IMAGE_DATA m_sData;
	P_GT_PROFILE_IMAGE_DATA m_sDataFromWAS;
	P_GT_PROFILE_IMAGE_DATA m_sDataDefault;

	INpStreamSession * m_pSession;
	CProfileImageActor *m_pProfileImageActor;

//#if defined(WIN32)
//	LONG m_nRef;
//#else //defined(WIN32)
//	volatile  int32_t m_nRef;
//#endif //defined(WIN32)
};
